@@ -1,15 +1,15 @@
 package main
 
 import (
-	"mlpack.org/v1/mlpack"
-	"testing"
-	"os"
+  "mlpack.org/v1/mlpack"
+  "testing"
+  "os"
 
-	"gonum.org/v1/gonum/mat"
+  "gonum.org/v1/gonum/mat"
 )
 
 func TestRunBindingNoFlag(t *testing.T) {
-	t.Log("Test that if we forget the mandatory flag, we should get wrong",
+  t.Log("Test that if we forget the mandatory flag, we should get wrong",
         "results.")
   param := mlpack.TestGoBindingOptions()
   d := 4.0
@@ -30,7 +30,7 @@ func TestRunBindingNoFlag(t *testing.T) {
 }
 
 func TestRunBindingCorrectly(t *testing.T) {
-	t.Log("Test that when we run the binding correctly (with correct",
+  t.Log("Test that when we run the binding correctly (with correct",
         " input parameters), we get the expected output.")
   param := mlpack.TestGoBindingOptions()
   param.Flag1 = true
@@ -313,6 +313,8 @@ func TestGonumRow(t *testing.T) {
   x := mat.NewDense(9, 1,  []float64{
     1, 2, 3, 4, 5, 6, 7, 8, 9,
   })
+  oldX := mat.NewDense(9, 1, nil)
+  oldX.Copy(x)
 
   param := mlpack.TestGoBindingOptions()
   param.RowIn = x
@@ -327,9 +329,9 @@ func TestGonumRow(t *testing.T) {
     t.Errorf("Error. Wrong shape.")
   }
   for i := 0; i < rows; i++ {
-    if RowOut.At(i, 0) != x.At(i, 0)*2 {
+    if RowOut.At(i, 0) != oldX.At(i, 0)*2 {
       val := RowOut.At(i, 0)
-      expected := x.At(i, 0) * 2
+      expected := oldX.At(i, 0) * 2
       t.Errorf("Error. Value at [i] : %v. Expected value : %v",
                val, expected)
     }
@@ -369,6 +371,8 @@ func TestGonumCol(t *testing.T) {
   x := mat.NewDense(1, 9, []float64{
     1, 2, 3, 4, 5, 6, 7, 8, 9,
   })
+  oldX := mat.NewDense(1, 9, nil)
+  oldX.Copy(x)
 
   param := mlpack.TestGoBindingOptions()
   param.ColIn = x
@@ -383,9 +387,9 @@ func TestGonumCol(t *testing.T) {
     t.Errorf("Error. Wrong shape.")
   }
   for i := 0; i < cols; i++ {
-    if ColOut.At(0, i) != x.At(0, i)*2 {
+    if ColOut.At(0, i) != oldX.At(0, i)*2 {
       val := ColOut.At(0, i)
-      expected := x.At(0, i) * 2
+      expected := oldX.At(0, i) * 2
       t.Errorf("Error. Value at [i] : %v. Expected value : %v",
                val, expected)
     }
@@ -549,6 +553,9 @@ func TestGonumMatrixWithInfo(t *testing.T) {
            11, 12, 13, 14, 15,
   })
 
+  oldX := mat.NewDense(3, 5, nil)
+  oldX.Copy(x.Data)
+
   param := mlpack.TestGoBindingOptions()
   param.MatrixAndInfoIn = x
   d := 4.0
@@ -564,9 +571,9 @@ func TestGonumMatrixWithInfo(t *testing.T) {
   }
   for i := 0; i < rows; i++ {
     for j := 0; j < cols; j++ {
-      if x.Data.At(i, j)*2 != MatrixAndInfoOut.At(i, j) {
+      if oldX.At(i, j)*2 != MatrixAndInfoOut.At(i, j) {
         val := MatrixAndInfoOut.At(i, j)
-        expected := x.Data.At(i, j)*2
+        expected := oldX.At(i, j)*2
         t.Errorf("Error. Value at [%v,%v] : %v. Expected value : %v",
                  i, j, val, expected)
       }
@@ -574,6 +581,61 @@ func TestGonumMatrixWithInfo(t *testing.T) {
   }
 }
 
+func TestGonumMatrixWithInfoCategorical(t *testing.T) {
+  t.Log("Test that the matrix with info option works when we pass categorical ",
+        "data.")
+
+  x := mlpack.DataAndInfo()
+  x.Categoricals = []bool{
+    false, false, true, true, false,
+  }
+
+  x.Data = mat.NewDense(6, 5, []float64{
+       0.1,  0.2, 3, 2, 0.3,
+       0.5, -0.3, 1, 1, 0.5,
+       -3,   0.1, 0, 0, 0.6,
+       0.7,  0.0, 2, 4, 0.4,
+       0.8,  0.1, 2, 3, 0.1,
+       0.3,  0.0, 1, 1, 0.6,
+  })
+
+  oldX := mat.NewDense(6, 5, nil)
+  oldX.Copy(x.Data)
+
+  param := mlpack.TestGoBindingOptions()
+  param.MatrixAndInfoIn = x
+  d := 4.0
+  i := 12
+  s := "hello"
+  _, _, _, MatrixAndInfoOut, _, _, _, _, _, _, _, _, _, _ :=
+      mlpack.TestGoBinding(d, i, s, param)
+
+  rows, cols := MatrixAndInfoOut.Dims()
+
+  if rows != 6 || cols != 5 {
+    t.Errorf("Error. Wrong shape. %v, %v", rows, cols)
+  }
+  for i := 0; i < rows; i++ {
+    for j := 0; j < cols; j++ {
+      if j == 0 || j == 1 || j == 4 {
+        if oldX.At(i, j) * 2 != MatrixAndInfoOut.At(i, j) {
+          val := MatrixAndInfoOut.At(i, j)
+          expected := oldX.At(i, j)*2
+          t.Errorf("Error. Value at [%v,%v] : %v. Expected value : %v",
+                   i, j, val, expected)
+        }
+      } else {
+        if oldX.At(i, j) != MatrixAndInfoOut.At(i, j) {
+          val := MatrixAndInfoOut.At(i, j)
+          expected := oldX.At(i, j)
+          t.Errorf("Error. Value at [%v,%v] : %v. Expected value: %v",
+                   i, j, val, expected)
+        }
+      }
+    }
+  }
+}
+
 func TestModel(t *testing.T) {
   t.Log("First create a GaussianKernel object, then send it back and",
         "make sure we get the right double value.")
